/*      $Id: lircd.c,v 5.46 2002/09/21 15:25:28 lirc Exp $      */

/****************************************************************************
 ** lircd.c *****************************************************************
 ****************************************************************************
 *
 * lircd - LIRC Decoder Daemon
 * 
 * Copyright (C) 1996,97 Ralph Metzler <rjkm@thp.uni-koeln.de>
 * Copyright (C) 1998,99 Christoph Bartelmus <lirc@bartelmus.de>
 *
 *  =======
 *  HISTORY
 *  =======
 *
 * 0.1:  03/27/96  decode SONY infra-red signals
 *                 create mousesystems mouse signals on pipe /dev/lircm
 *       04/07/96  send ir-codes to clients via socket (see irpty)
 *       05/16/96  now using ir_remotes for decoding
 *                 much easier now to describe new remotes
 *
 * 0.5:  09/02/98 finished (nearly) complete rewrite (Christoph)
 *
 */

/* disable daemonise if maintainer mode SIM_REC / SIM_SEND defined */
#if defined(SIM_REC) || defined (SIM_SEND)
# undef DAEMONIZE
#endif

#define _GNU_SOURCE
#define _BSD_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <getopt.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <fcntl.h>
#include <sys/file.h>

#ifndef timersub
#define timersub(a, b, result)                                                \
do {                                                                        \
   (result)->tv_sec = (a)->tv_sec - (b)->tv_sec;                             \
      (result)->tv_usec = (a)->tv_usec - (b)->tv_usec;                          \
      if ((result)->tv_usec < 0) {                                              \
         --(result)->tv_sec;                                                     \
            (result)->tv_usec += 1000000;                                           \
      }                                                                         \
} while (0)
#endif

#include "lircd.h"
#include "ir_remote.h"
#include "config_file.h"
#include "hardware.h"
#include "hw-types.h"

#include <BeBuild.h>

_EXPORT int lirc_init(char *device, char *driver);
_EXPORT int lirc_init_device(void);
_EXPORT int lirc_deinit(void);
_EXPORT char *lirc_read_data(void);
_EXPORT int lirc_add_remote(char *configfile);
_EXPORT int lirc_del_remote(char *configfile);

struct ir_remote *remotes;
struct ir_remote *free_remotes=NULL;

extern struct ir_remote *decoding;
extern struct ir_remote *last_remote;
extern struct ir_remote *repeat_remote;
extern struct ir_ncode *repeat_code;

extern struct hardware hw;

#define VERSION "1.0."
#define LOGFILE "/var/log/lirc"

char *progname="lircd " VERSION;
char *logfile=LOGFILE;
FILE *pidfile;

char gsz_Driver[512];

FILE *lf=NULL;

/* fixme: */
#define MAX_PEERS	100


#define CT_LOCAL  1
#define CT_REMOTE 2

int debug=0;
int daemonized=0;


void logprintf(int prio,char *format_str, ...)
{
   time_t current;
   char *currents;
   va_list ap;  

   current=time(&current);
   currents=ctime(&current);

   if(lf) fprintf(lf,"%15.15s %s: ",currents+4,progname);
   if(!daemonized) fprintf(stderr,"%s: ",progname);
   va_start(ap,format_str);
   if(lf)
   {
      if(prio==LOG_WARNING) fprintf(lf,"WARNING: ");
      vfprintf(lf,format_str,ap);
      fputc('\n',lf);fflush(lf);
   }
   if(!daemonized)
   {
      if(prio==LOG_WARNING) fprintf(stderr,"WARNING: ");
      vfprintf(stderr,format_str,ap);
      fputc('\n',stderr);fflush(stderr);
   }
   va_end(ap);
}

void logperror(int prio,const char *s)
{
   if(s!=NULL)
   {
      logprintf(prio,"%s: %s",s,strerror(errno));
   }
   else
   {
      logprintf(prio,"%s",strerror(errno));
   }
}

int waitfordata(long maxusec)
{
   if( ioctl(hw.fd,LIRC_WAIT_DATA,&maxusec)==-1)
   {
      logprintf(LOG_ERR,"GET_READ_STATUS failed\n");
      return(0);
   }
   if(maxusec == 1)
      return(1);
   
   return(0);
}

char *lirc_read_data(void)
{
   char *message = NULL;

   if(waitfordata(10000))
   {
      if(hw.rec_func != NULL)
         message=hw.rec_func(remotes);
   }
   /*
   if(message)
   {
      char logstring[255];
      sprintf(logstring, "received key %s", message);
      logprintf(LOG_NOTICE, logstring);
   }
   */
   return(message);
}

int lirc_init(char *device, char *driver)
{
   hw_choose_driver(device);

   if(strcmp(hw.name, "null")==0)
   {
      fprintf(stderr,"%s: there's no hardware I can use and "
            "no peers are specified\n",progname);
      return(EXIT_FAILURE);
   }

   if(driver != NULL)
      strcpy(gsz_Driver, driver);
   hw.device = gsz_Driver;
   remotes=NULL;

   lf=fopen(logfile,"a");
   if(lf==NULL)
   {
      fprintf(stderr,"%s: could not open logfile\n",progname);
      perror(progname);
      return(EXIT_FAILURE);
   }
   LOGPRINTF(1,"started server socket");
   logprintf(LOG_NOTICE,"lircd ready");

   return(0); 
}

int lirc_init_device(void)
{
   if(hw.init_func)
   {
      if(!hw.init_func()) 
         return(-1);
   }
   return(0);
}

int lirc_deinit(void)
{
   hw.deinit_func();

   fclose(lf);
   return(0);
}

int lirc_add_remote(char *configfile)
{
   FILE *fd;
   struct ir_remote *config_remotes;

   if(free_remotes!=NULL)
   {
      logprintf(LOG_ERR,"cannot read config file");
      logprintf(LOG_ERR,"old config is still in use");
      return(-1);
   }
   fd=fopen(configfile,"r");
   if(fd==NULL)
   {
      logprintf(LOG_ERR,"could not open config file '%s'",
            configfile);
      logperror(LOG_ERR,NULL);
      return(-1);
   }
   config_remotes=read_config(fd);
   fclose(fd);
   if(config_remotes==(void *) -1)
   {
      logprintf(LOG_ERR,"reading of config file failed");
   }
   else
   {
      LOGPRINTF(1,"config file read");
      if(config_remotes==NULL)
      {
         logprintf(LOG_WARNING,"config file contains no "
               "valid remote control definition");
      }
      /* I cannot free the data structure
         as they could still be in use */
      free_remotes=remotes;
      remotes=config_remotes;
      if(hw.config_func) (void) hw.config_func(remotes);
   }
   return(0);
}


int lirc_del_remote(char *configfile)
{
   return(0);
}


